@@ -1,17 +1,17 @@
-from typing import Callable, List, Optional, Dict, Any
+from typing import Callable, List, Optional, Dict, Any, Type, Union
 
 import numpy as np
 import numpy.typing as npt
 import inspect
 from typeguard import typechecked
-
+import tensorflow as tf  # type: ignore
 
 from code_loader.contract.datasetclasses import SectionCallableInterface, InputHandler, \
     GroundTruthHandler, MetadataHandler, DatasetIntegrationSetup, VisualizerHandler, PreprocessResponse, \
     PreprocessHandler, VisualizerCallableInterface, CustomLossHandler, CustomCallableInterface, PredictionTypeHandler, \
-    MetadataSectionCallableInterface, UnlabeledDataPreprocessHandler
+    MetadataSectionCallableInterface, UnlabeledDataPreprocessHandler, CustomLayerHandler, \
+    ConfusionMatrixCallableInterface
 from code_loader.contract.enums import DatasetMetadataType, LeapDataType, Metric
-from code_loader.contract.visualizer_classes import validate_type
 from code_loader.visualizers.default_visualizers import DefaultVisualizer, \
     default_graph_visualizer, \
     default_image_visualizer, default_horizontal_bar_visualizer, default_word_visualizer, \
@@ -78,8 +78,10 @@ class LeapBinder:
         self.setup_container.custom_loss_handlers.append(CustomLossHandler(name, function))
 
     @typechecked
-    def add_prediction(self, name: str, labels: List[str], metrics: List[Metric],
-                       custom_metrics: Optional[List[CustomCallableInterface]] = None) -> None:
+    def add_prediction(
+            self, name: str, labels: List[str], metrics: List[Metric],
+            custom_metrics: Optional[List[Union[CustomCallableInterface, ConfusionMatrixCallableInterface]]] = None
+    ) -> None:
         self.setup_container.prediction_types.append(PredictionTypeHandler(name, labels, metrics, custom_metrics))
 
     @typechecked
@@ -93,3 +95,10 @@ class LeapBinder:
     def set_metadata(self, function: MetadataSectionCallableInterface, metadata_type: DatasetMetadataType,
                      name: str) -> None:
         self.setup_container.metadata.append(MetadataHandler(name, function, metadata_type))
+
+    @typechecked
+    def set_custom_layer(self, custom_layer: Type[tf.keras.layers.Layer], name: str) -> None:
+        init_args = inspect.getfullargspec(custom_layer.__init__)[0][1:]
+        call_args = inspect.getfullargspec(custom_layer.call)[0][1:]
+        self.setup_container.custom_layers[name] = CustomLayerHandler(name, custom_layer, init_args, call_args)
+
